@@ -29,6 +29,7 @@
 # include <windows.h>
 #include <conio.h>
 #include <tlhelp32.h>
+#include <direct.h>
 #endif
 
 #if defined(UNIX)
@@ -74,6 +75,9 @@
 #include <IOKit/IOBSD.h>
 #include <mach/mach_init.h>
 #include <mach/task_policy.h>
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1050
+# include <AudioToolbox/AudioServices.h>
+#endif
 #endif /* __APPLE__ */
 
 #undef DEBUG
@@ -118,8 +122,6 @@ int next_con_char(void) {
 				return buf[0];
 			}
 		}
-
-		return 0;
 	}
 
 	c = _getch();
@@ -206,10 +208,43 @@ void msec_sleep(unsigned int msec) {
 }
 
 /* Return the current time in msec since */
-/* the process started. */
+/* the first invokation of msec_time() */
 /* (Is this based on timeGetTime() ? ) */
 unsigned int msec_time() {
-	return GetTickCount();
+	unsigned int rv;
+	static unsigned int startup = 0;
+
+	rv =  GetTickCount();
+	if (startup == 0)
+		startup = rv;
+
+	return rv - startup;
+}
+
+/* Return the current time in usec */
+/* since the first invokation of usec_time() */
+/* Return -1.0 if not available */
+double usec_time() {
+	double rv;
+	LARGE_INTEGER val;
+	static double scale = 0.0;
+	static LARGE_INTEGER startup;
+
+	if (scale == 0.0) {
+		if (QueryPerformanceFrequency(&val) == 0)
+			return -1.0;
+		scale = 1000000.0/val.QuadPart;
+		QueryPerformanceCounter(&val);
+		startup.QuadPart = val.QuadPart;
+
+	} else {
+		QueryPerformanceCounter(&val);
+	}
+	val.QuadPart -= startup.QuadPart;
+
+	rv = val.QuadPart * scale;
+		
+	return rv;
 }
 
 static athread *beep_thread = NULL;
@@ -239,6 +274,8 @@ void msec_beep(int delay, int freq, int msec) {
 	}
 }
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 #ifdef NEVER    /* Not currently needed, or effective */
 
 /* Set the current threads priority */
@@ -259,6 +296,17 @@ int set_normal_priority() {
 
 #undef USE_BEGINTHREAD
 
+/* Wait for the thread to exit. Return the result */
+static int athread_wait(struct _athread *p) {
+
+	if (p->finished)
+		return p->result;
+
+	WaitForSingleObject(p->th, INFINITE);
+
+	return p->result;
+}
+
 /* Destroy the thread */
 static void athread_del(
 athread *p
@@ -268,9 +316,11 @@ athread *p
 	if (p == NULL)
 		return;
 
-	if (p->th != NULL) {		/* Oops. this isn't good. */
-		DBG("athread_del calling TerminateThread() because thread hasn't finished\n");
-		TerminateThread(p->th, -1);		/* But it is worse to leave it hanging around */
+	if (p->th != NULL) {
+		if (!p->finished) {
+			DBG("athread_del calling TerminateThread() because thread hasn't finished\n");
+			TerminateThread(p->th, (DWORD)-1);		/* But it is worse to leave it hanging around */
+		}
 		CloseHandle(p->th);
 	}
 
@@ -292,8 +342,7 @@ DWORD WINAPI threadproc(
 	athread *p = (athread *)lpParameter;
 
 	p->result = p->function(p->context);
-	CloseHandle(p->th);
-	p->th = NULL;
+	p->finished = 1;
 #ifdef USE_BEGINTHREAD
 #else
 	return 0;
@@ -313,6 +362,7 @@ athread *new_athread(
 
 	p->function = function;
 	p->context = context;
+	p->wait = athread_wait;
 	p->del = athread_del;
 
 	/* Create a thread */
@@ -324,6 +374,7 @@ athread *new_athread(
 	if (p->th == NULL) {
 #endif
 		DBG("Failed to create thread\n");
+		p->th = NULL;
 		athread_del(p);
 		return NULL;
 	}
@@ -332,6 +383,8 @@ athread *new_athread(
 	return p;
 }
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 /* Delete a file */
 void delete_file(char *fname) {
 	_unlink(fname);
@@ -484,7 +537,8 @@ void msec_sleep(unsigned int msec) {
 #endif
 }
 
-/* Return the current time in msec. This is not related to any particular epoch */
+/* Return the current time in msec */
+/* since the first invokation of msec_time() */
 unsigned int msec_time() {
 	unsigned int rv;
 	static struct timeval startup = { 0, 0 };
@@ -514,6 +568,37 @@ unsigned int msec_time() {
 	return rv;
 }
 
+/* Return the current time in usec */
+/* since the first invokation of usec_time() */
+double usec_time() {
+	double rv;
+	static struct timeval startup = { 0, 0 };
+	struct timeval cv;
+
+	/* Is this monotonic ? */
+	/* On Linux, should clock_gettime with CLOCK_MONOTONIC/CLOCK_REALTIME/CLOCK_REALTIME_HR ? */
+	/* On OS X, should mach_absolute_time() be used ? */
+	/* or host_get_clock_service(mach_host_self(), SYSTEM_CLOCK, &clk) */
+	gettimeofday(&cv, NULL);
+
+	/* Set time to 0 on first invocation */
+	if (startup.tv_sec == 0 && startup.tv_usec == 0)
+		startup = cv;
+
+	/* Subtract, taking care of carry */
+	cv.tv_sec -= startup.tv_sec;
+	if (startup.tv_usec > cv.tv_usec) {
+		cv.tv_sec--;
+		cv.tv_usec += 1000000;
+	}
+	cv.tv_usec -= startup.tv_usec;
+
+	/* Convert to usec */
+	rv = cv.tv_sec * 1000000.0 + cv.tv_usec;
+
+	return rv;
+}
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #ifdef NEVER    /* Not currently needed, or effective */
@@ -607,7 +692,11 @@ static int beep_msec;
 static int delayed_beep(void *pp) {
 	msec_sleep(beep_delay);
 #ifdef __APPLE__
+# if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1050
+	AudioServicesPlayAlertSound(kUserPreferredAlert);
+# else
 	SysBeep((beep_msec * 60)/1000);
+# endif
 #else
 	fprintf(stdout, "\a"); fflush(stdout);
 #endif 
@@ -626,7 +715,11 @@ void msec_beep(int delay, int freq, int msec) {
 			error("Delayed beep failed to create thread");
 	} else {
 #ifdef __APPLE__
-		SysBeep((msec * 60)/1000);
+# if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1050
+			AudioServicesPlayAlertSound(kUserPreferredAlert);
+# else
+			SysBeep((msec * 60)/1000);
+# endif
 #else
 		/* Linux is pretty lame in this regard... */
 		fprintf(stdout, "\a"); fflush(stdout);
@@ -684,6 +777,17 @@ XBell(..);
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - */
 
+/* Wait for the thread to exit. Return the result */
+static int athread_wait(struct _athread *p) {
+
+	if (p->finished)
+		return p->result;
+
+	pthread_join(p->thid, NULL);
+
+	return p->result;
+}
+
 /* Destroy the thread */
 static void athread_del(
 athread *p
@@ -693,7 +797,9 @@ athread *p
 	if (p == NULL)
 		return;
 
-	pthread_cancel(p->thid);
+	if (!p->finished) {
+		pthread_cancel(p->thid);
+	}
 	pthread_join(p->thid, NULL);
 
 	free(p);
@@ -705,6 +811,8 @@ static void *threadproc(
 	athread *p = (athread *)param;
 
 	p->result = p->function(p->context);
+	p->finished = 1;
+
 	return 0;
 }
  
@@ -722,6 +830,7 @@ athread *new_athread(
 
 	p->function = function;
 	p->context = context;
+	p->wait = athread_wait;
 	p->del = athread_del;
 
 	/* Create a thread */
@@ -983,87 +1092,6 @@ int kill_nprocess(char **pname, int debug) {
 
 #endif /* __APPLE__ || NT */
 
-/* ============================================================= */
-/* Provide a system independent glob type function */
-
-/* Create the aglob */
-/* Return nz on malloc error */
-int aglob_create(aglob *g, char *spath) {
-#ifdef NT
-	char *pp;
-	int rlen;
-	/* Figure out where the filename starts */
-	if ((pp = strrchr(spath, '/')) == NULL
-	 && (pp = strrchr(spath, '\\')) == NULL)
-		rlen = 0;
-	else
-		rlen = pp - spath + 1;
-
-	if ((g->base = malloc(rlen + 1)) == NULL)
-		return 1;
-
-	memmove(g->base, spath, rlen);
-	g->base[rlen] = '\000';
-
-	g->first = 1;
-    g->ff = _findfirst(spath, &g->ffs);
-#else /* UNIX */
-	g->rv = glob(spath, GLOB_NOSORT, NULL, &g->g);
-	if (g->rv == GLOB_NOSPACE)
-		return 1;
-	g->ix = 0;
-#endif
-	g->merr = 0;
-	return 0;
-}
-
-/* Return an allocated string of the next match. */
-/* Return NULL if no more matches */
-char *aglob_next(aglob *g) {
-	char *fpath;
-
-#ifdef NT
-	if (g->ff == -1L) {
-		return NULL;
-	}
-	if (g->first == 0) {
-		if (_findnext(g->ff, &g->ffs) != 0) {
-			return NULL;
-		}
-	}
-	g->first = 0;
-
-	/* Convert match filename to full path */
-	if ((fpath = malloc(strlen(g->base) + strlen(g->ffs.name) + 1)) == NULL) {
-		g->merr = 1;
-		return NULL;
-	}
-	strcpy(fpath, g->base);
-	strcat(fpath, g->ffs.name);
-	return fpath;
-#else
-	if (g->rv != 0 || g->ix >= g->g.gl_pathc)
-		return NULL;
-	if ((fpath = strdup(g->g.gl_pathv[g->ix])) == NULL) {
-		g->merr = 1;
-		return NULL;
-	}
-	g->ix++;
-	return fpath;
-#endif
-}
-
-void aglob_cleanup(aglob *g) {
-#ifdef NT
-	if (g->ff != -1L)
-		_findclose(g->ff);
-	free(g->base);
-#else /* UNIX */
-	if (g->rv == 0)
-		globfree(&g->g);
-#endif
-}
-
 /* ============================================================= */
 /* CCSS location support */
 
@@ -1330,6 +1358,202 @@ void sa_Scale3(double out[3], double in[3], double rat) {
 	out[2] = in[2] * rat;
 }
 
+/* Return the normal Delta E given two Lab values */
+double sa_LabDE(double *Lab0, double *Lab1) {
+	double rv = 0.0, tt;
+
+	tt = Lab0[0] - Lab1[0];
+	rv += tt * tt;
+	tt = Lab0[1] - Lab1[1];
+	rv += tt * tt;
+	tt = Lab0[2] - Lab1[2];
+	rv += tt * tt;
+
+	return sqrt(rv);
+}
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - */
+/* A sub-set of ludecomp code from numlib          */
+
+int sa_lu_decomp(double **a, int n, int *pivx, double *rip) {
+	int    i, j;
+	double *rscale, RSCALE[10];		
+
+	if (n <= 10)
+		rscale = RSCALE;
+	else
+		rscale = dvector(0, n-1);
+
+	for (i = 0; i < n; i++) {
+		double big;
+		for (big = 0.0, j=0; j < n; j++) {
+			double temp;
+			temp = fabs(a[i][j]);
+			if (temp > big)
+				big = temp;
+		}
+		if (fabs(big) <= DBL_MIN) {
+			if (rscale != RSCALE)
+				free_dvector(rscale, 0, n-1);
+			return 1;		
+		}
+		rscale[i] = 1.0/big;	
+	}
+
+	for (*rip = 1.0, j = 0; j < n; j++) {
+		double big;
+		int k, bigi = 0;
+
+		for (i = 0; i < j; i++) {
+			double sum;
+			sum = a[i][j];
+			for (k = 0; k < i; k++)
+				sum -= a[i][k] * a[k][j];
+			a[i][j] = sum;
+		}
+
+		for (big = 0.0, i = j; i < n; i++) {
+			double sum, temp;
+			sum = a[i][j];
+			for (k = 0; k < j; k++)
+				sum -= a[i][k] * a[k][j];
+			a[i][j] = sum;
+			temp = rscale[i] * fabs(sum);	
+			if (temp >= big) {
+				big = temp;		
+				bigi = i;		
+			}
+		}
+		
+		if (j != bigi) {
+			{	
+				double *temp;
+				temp = a[bigi];
+				a[bigi] = a[j];
+				a[j] = temp;
+			}
+			*rip = -(*rip);				
+			rscale[bigi] = rscale[j];	
+		}
+		
+		pivx[j] = bigi;					
+		if (fabs(a[j][j]) <= DBL_MIN) {
+			if (rscale != RSCALE)
+				free_dvector(rscale, 0, n-1);
+			return 1; 					
+		}
+
+		if (j != (n-1)) {
+			double temp;
+			temp = 1.0/a[j][j];
+			for (i = j+1; i < n; i++)
+				a[i][j] *= temp;
+		}
+	}
+	if (rscale != RSCALE)
+		free_dvector(rscale, 0, n-1);
+	return 0;
+}
+
+void sa_lu_backsub(double **a, int n, int *pivx, double  *b) {
+	int i, j;
+	int nvi;		
+	
+	for (nvi = -1, i = 0; i < n; i++) {
+		int px;
+		double sum;
+
+		px = pivx[i];
+		sum = b[px];
+		b[px] = b[i];
+		if (nvi >= 0) {
+			for (j = nvi; j < i; j++)
+				sum -= a[i][j] * b[j];
+		} else {
+			if (sum != 0.0)
+				nvi = i;			
+		}
+		b[i] = sum;
+	}
+
+	for (i = (n-1); i >= 0; i--) {
+		double sum;
+		sum = b[i];
+		for (j = i+1; j < n; j++)
+			sum -= a[i][j] * b[j];
+		b[i] = sum/a[i][i];
+	}
+}
+
+int sa_lu_invert(double **a, int n) {
+	int i, j;
+	double rip;		
+	int *pivx, PIVX[10];
+	double **y;
+
+	if (n <= 10)
+		pivx = PIVX;
+	else
+		pivx = ivector(0, n-1);
+
+	if (sa_lu_decomp(a, n, pivx, &rip)) {
+		if (pivx != PIVX)
+			free_ivector(pivx, 0, n-1);
+		return 1;
+	}
+
+	y = dmatrix(0, n-1, 0, n-1);
+	for (i = 0; i < n; i++) {
+		for (j = 0; j < n; j++) {
+			y[i][j] = a[i][j];
+		}
+	}
+
+	for (i = 0; i < n; i++) {
+		for (j = 0; j < n; j++)
+			a[i][j] = 0.0;
+		a[i][i] = 1.0;
+		sa_lu_backsub(y, n, pivx, a[i]);
+	}
+
+	free_dmatrix(y, 0, n-1, 0, n-1);
+	if (pivx != PIVX)
+		free_ivector(pivx, 0, n-1);
+
+	return 0;
+}
+
+int sa_lu_psinvert(double **out, double **in, int m, int n) {
+	int rv = 0;
+	double **tr;		
+	double **sq;		
+
+	tr = dmatrix(0, n-1, 0, m-1);
+	matrix_trans(tr, in, m,  n);
+
+	if (m > n) {
+		sq = dmatrix(0, n-1, 0, n-1);
+		if ((rv = matrix_mult(sq, n, n, tr, n, m, in, m, n)) == 0) {
+			if ((rv = sa_lu_invert(sq, n)) == 0) {
+				rv = matrix_mult(out, n, m, sq, n, n, tr, n, m);
+			}
+		}
+		free_dmatrix(sq, 0, n-1, 0, n-1);
+	} else {
+		sq = dmatrix(0, m-1, 0, m-1);
+		if ((rv = matrix_mult(sq, m, m, in, m, n, tr, n, m)) == 0) {
+			if ((rv = sa_lu_invert(sq, m)) == 0) {
+				rv = matrix_mult(out, n, m, tr, n, m, sq, m, m);
+			}
+		}
+		free_dmatrix(sq, 0, m-1, 0, m-1);
+	}
+
+	free_dmatrix(tr, 0, n-1, 0, m-1);
+	return rv;
+}
+
+
 #endif /* SALONEINSTLIB */
 /* ============================================================= */
 
