@@ -133,7 +133,7 @@ struct _icoms *p
 		PSP_DEVICE_INTERFACE_DETAIL_DATA pdidd = (PSP_DEVICE_INTERFACE_DETAIL_DATA)buf;
 		SP_DEVINFO_DATA dinfod;
 		int i;
-		unsigned short VendorID, ProductID;
+		unsigned short VendorID = 0, ProductID = 0;
 	
 		/* Make sure we've dynamically linked */
 		if (setup_dyn_calls() == 0) {
@@ -180,7 +180,7 @@ struct _icoms *p
 						break;
 					memcpy(buf, cp + 4, 4);
 					buf[4] = '\000';
-					if (sscanf(buf, "%x", &VendorID) != 1)
+					if (sscanf(buf, "%hx", &VendorID) != 1)
 						break;
 					if ((cp = strchr(pdidd->DevicePath, 'p')) == NULL)
 						break;
@@ -190,7 +190,7 @@ struct _icoms *p
 						break;
 					memcpy(buf, cp + 4, 4);
 					buf[4] = '\000';
-					if (sscanf(buf, "%x", &ProductID) != 1)
+					if (sscanf(buf, "%hx", &ProductID) != 1)
 						break;
 					gotid = 1;
 					break;
@@ -248,10 +248,6 @@ struct _icoms *p
 #endif /* NT */
 
 #ifdef __APPLE__
-	/* On OS X the general USB driver will locate the HID devices, but */
-	/* not be able to claim the interfaces without using a kext to stop */
-	/* the HID driver grabing the interfaces. We switch them from libusb */
-	/* access to OS X HID driver access to avoid the problem. */
 	{
 	    kern_return_t kstat; 
 	    CFMutableDictionaryRef sdict;		/* HID Device  dictionary */
@@ -262,7 +258,7 @@ struct _icoms *p
         	warning("IOServiceMatching returned a NULL dictionary");
 		}
 
-		/* Init itterator to find matching types */
+		/* Init itterator to find matching types. Consumes sdict reference */
 		if ((kstat = IOServiceGetMatchingServices(kIOMasterPortDefault, sdict, &mit))
 			                                                          != KERN_SUCCESS) 
         	error("IOServiceGetMatchingServices returned %d", kstat);
@@ -270,50 +266,63 @@ struct _icoms *p
 		/* Find all the matching HID devices */
 		for (;;) {
 			io_object_t ioob;						/* HID object found */
-		    CFMutableDictionaryRef hidprops = 0;	/* HID Device properties */
-			CFNumberRef vref, pref;
+			CFNumberRef vref, pref;					/* HID Vendor and Product ID propeties */
 			unsigned int vid = 0, pid = 0;
+			instType itype;
 
 		    if ((ioob = IOIteratorNext(mit)) == 0)
 				break;
 
-			/* Get the devices vid and pid */
-			if (IORegistryEntryCreateCFProperties(ioob, &hidprops,
-					kCFAllocatorDefault, kNilOptions) != KERN_SUCCESS || hidprops == 0)
-				goto continue1;
-
-			if ((vref = CFDictionaryGetValue(hidprops, CFSTR(kIOHIDVendorIDKey))) != 0) {
+			/* Get the two properies we need. [ Doing IORegistryEntryCreateCFProperty() is much faster */
+			/* than IORegistryEntryCreateCFProperties() in some cases.] */
+			if ((vref = IORegistryEntryCreateCFProperty(ioob, CFSTR(kIOHIDVendorIDKey),
+			                                         kCFAllocatorDefault,kNilOptions)) != 0) {
 				CFNumberGetValue(vref, kCFNumberIntType, &vid);
-				CFRelease(vref);
+			    CFRelease(vref);
 			}
-			if ((pref = CFDictionaryGetValue(hidprops, CFSTR(kIOHIDProductIDKey))) != 0) {
+			if ((pref = IORegistryEntryCreateCFProperty(ioob, CFSTR(kIOHIDProductIDKey),
+			                                         kCFAllocatorDefault,kNilOptions)) != 0) {
 				CFNumberGetValue(pref, kCFNumberIntType, &pid);
-				CFRelease(pref);
+			    CFRelease(pref);
 			}
-// ~~999
-//			CFRelease(hidprops);		// Crashes !!
-			hidprops = NULL;
 
 			/* If it's a device we're looking for */
-			if (inst_usb_match(vid, pid) != instUnknown) {
-				int i;
-				for (i = 0; i < p->npaths; i++) {	/* Locate it in the current list */
-					if (p->paths[i]->dev != NULL
-					 && p->paths[i]->vid == vid && p->paths[i]->pid == pid) {
+			if ((itype = inst_usb_match(vid, pid)) != instUnknown) {
+				char pname[400];
+				if (p->debug) fprintf(stderr,"found HID device '%s' that we're looking for\n",inst_name(itype));
 
-						if ((p->paths[i]->hev = (hid_device *)calloc(sizeof(hid_device), 1)) == NULL)
-							error("icoms: calloc failed!");
-
-						p->paths[i]->hev->ioob = ioob;
-						usb_del_usb_device(p->paths[i]->dev);	/* Done with this */
-						p->paths[i]->dev = NULL;	/* (Points to libusb allocation) */
-						ioob = 0;
-						break;
-					}
+				/* Create human readable path/identification */
+				/* (There seems to be no easy way of creating an interface no, without */
+				/*  heroic efforts looking through the IO registry, so don't try.) */
+				sprintf(pname,"hid: (%s)", inst_name(itype));
+		
+				/* Add the path to the list */
+				if (p->paths == NULL) {
+					if ((p->paths = (icompath **)calloc(sizeof(icompath *), 1 + 1)) == NULL)
+						error("icoms: calloc failed!");
+				} else {
+					if ((p->paths = (icompath **)realloc(p->paths,
+					                     sizeof(icompath *) * (p->npaths + 2))) == NULL)
+						error("icoms: realloc failed!");
+					p->paths[p->npaths+1] = NULL;
 				}
+				if ((p->paths[p->npaths] = calloc(sizeof(icompath), 1)) == NULL)
+					error("icoms: calloc failed!");
+				p->paths[p->npaths]->vid = vid;
+				p->paths[p->npaths]->pid = pid;
+				p->paths[p->npaths]->dev = NULL;		/* Make sure it's NULL */
+				if ((p->paths[p->npaths]->hev = (hid_device *)calloc(sizeof(hid_device), 1))
+				                                                                     == NULL)
+					error("icoms: calloc failed!");
+				p->paths[p->npaths]->hev->ioob = ioob;
+				ioob = 0;			/* Don't release it */
+				p->paths[p->npaths]->itype = itype;
+				if ((p->paths[p->npaths]->path = strdup(pname)) == NULL)
+					error("icoms: strdup failed!");
+				p->npaths++;
+				p->paths[p->npaths] = NULL;
 			}
-		continue1:
-			if (ioob != 0)
+			if (ioob != 0)		/* If we haven't kept it */
 			    IOObjectRelease(ioob);		/* Release found object */
 		}
 	    IOObjectRelease(mit);			/* Release the itterator */
@@ -437,8 +446,6 @@ void hid_close_port(icoms *p) {
 	    IOObjectRelease(p->hidd->port);
 		p->hidd->port = 0;
 
-		CFRunLoopRemoveSource(p->hidd->rlr, p->hidd->evsrc, kCFRunLoopDefaultMode);
-
 		CFRelease(p->hidd->evsrc);
 		p->hidd->evsrc = NULL;
 
@@ -482,7 +489,7 @@ char **pnames			/* List of process names to try and kill before opening */
 
 	if (port >= 1) {
 		if (p->is_open && port != p->port) {	/* If port number changes */
-			icoms_close_port(p);
+			p->close_port(p);
 		}
 	}
 
@@ -576,11 +583,6 @@ char **pnames			/* List of process names to try and kill before opening */
 			        != kIOReturnSuccess || p->hidd->evsrc == NULL)
 				error("Creating event source on HID device '%s' failed",p->ppath->path);
 
-//			p->hidd->rlr = CFRetain(CFRunLoopGetCurrent());
-			p->hidd->rlr = CFRunLoopGetCurrent();
-
-			CFRunLoopAddSource(p->hidd->rlr, p->hidd->evsrc, kCFRunLoopDefaultMode);
-
 		}
 #endif /* __APPLE__ */
 
@@ -604,7 +606,12 @@ void *target,
 IOReturn result,
 void *refcon,
 void *sender,
-UInt32 size) {
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1050
+uint32_t size
+#else
+UInt32 size
+#endif
+) {
 	icoms *p = (icoms *)target;
 
 //printf("\n~1 callback called with size %d, result 0x%x\n",size,result);
@@ -682,7 +689,10 @@ icoms_hid_read_th(icoms *p,
 			error("Setting callback handler for HID '%s' failed", p->ppath->path);
 		if ((*(p->hidd->device))->startAllQueues(p->hidd->device) != kIOReturnSuccess)
 			error("Starting queues for HID '%s' failed", p->ppath->path);
+
 		/* Call runloop, but exit after handling one callback */
+		p->hidd->rlr = CFRunLoopGetCurrent();
+		CFRunLoopAddSource(p->hidd->rlr, p->hidd->evsrc, kCFRunLoopDefaultMode);
 		result = CFRunLoopRunInMode(kCFRunLoopDefaultMode, tout, false);
 		if ((*(p->hidd->device))->stopAllQueues(p->hidd->device) != kIOReturnSuccess)
 			error("Stopping queues for HID '%s' failed", p->ppath->path);
@@ -691,6 +701,8 @@ icoms_hid_read_th(icoms *p,
 		} else if (result != kCFRunLoopRunStopped) {
 			lerr = ICOM_USBR; 
 		}
+		CFRunLoopRemoveSource(p->hidd->rlr, p->hidd->evsrc, kCFRunLoopDefaultMode);
+
 		if (p->hidd->result == -1) {		/* Callback wasn't called */
 			lerr = ICOM_TO; 
 		} else if (p->hidd->result != kIOReturnSuccess) {
@@ -767,7 +779,7 @@ icoms_hid_write_th(icoms *p,
 		if ((wbuf2 = malloc(bsize + 1)) == NULL)
 			error("icoms_hid_write, malloc failed");
 		memmove(wbuf2+1,wbuf,bsize);
-		wbuf2[0] = 0;		/* Extra report ID byte */
+		wbuf2[0] = 0;		/* Extra report ID byte (why ?) */
 		if (WriteFile(p->hidd->fh, wbuf2, bsize+1, (LPDWORD)&bwritten, &p->hidd->ols) == 0) { 
 			if (GetLastError() != ERROR_IO_PENDING) {
 				lerr = ICOM_USBW; 
@@ -864,7 +876,7 @@ char **pnames			/* List of process names to try and kill before opening */
 	if (p->debug) fprintf(stderr,"icoms: About to set hid port characteristics\n");
 
 	if (p->is_open) 
-		icoms_close_port(p);
+		p->close_port(p);
 
 	if (p->is_hid_portno(p, port) != instUnknown) {
 
@@ -888,6 +900,8 @@ icoms *p
 ) {
 	p->is_hid_portno  = hid_is_hid_portno;
 	p->set_hid_port   = icoms_set_hid_port;
+	p->hid_read_th    = icoms_hid_read_th;
+	p->hid_write_th   = icoms_hid_write_th;
 	p->hid_read       = icoms_hid_read;
 	p->hid_write      = icoms_hid_write;
 
