@@ -58,11 +58,13 @@
 #undef DEBUG
 
 #ifdef DEBUG
-#define DBG(xxx) printf xxx ;
+#define DBG(xxx) fprintf xxx ;
 #else
-#define DBG(xxx) 
+#define DBG(xxx) if (p->debug >= 5) fprintf xxx ;
 #endif
 
+#define dbgo stderr
+
 static inst_code huey_interp_code(inst *pp, int ec);
 static inst_code huey_check_unlock(huey *p);
 
@@ -75,17 +77,20 @@ static inst_code huey_check_unlock(huey *p);
 /* Implementation */
 
 /* Interpret an icoms error into a HUEY error */
-static int icoms2huey_err(int se) {
+/* If torc is nz, then a trigger or command is OK, */
+/* othewise  they are treated as an abort. */
+static int icoms2huey_err(int se, int torc) {
 	if (se & ICOM_USERM) {
 		se &= ICOM_USERM;
-		if (se == ICOM_USER)
-			return HUEY_USER_ABORT;
+		if (torc) {
+			if (se == ICOM_TRIG)
+				return HUEY_USER_TRIG;
+			if (se == ICOM_CMND)
+				return HUEY_USER_CMND;
+		}
 		if (se == ICOM_TERM)
 			return HUEY_USER_TERM;
-		if (se == ICOM_TRIG)
-			return HUEY_USER_TRIG;
-		if (se == ICOM_CMND)
-			return HUEY_USER_CMND;
+		return HUEY_USER_ABORT;
 	}
 	if (se != ICOM_OK)
 		return HUEY_COMS_FAIL;
@@ -112,7 +117,7 @@ typedef enum {
     i1d_unlock       = 0x0e,		/* BBBB:-      Unlock the interface */
     i1d_m_red_2      = 0x13,		/* B:2:W       Measure the red channel in freq mode, 1,10sec */
 									/* B = sync mode, typically 2 */
-    i1d_m_rgb_edge_2 = 0x16,		/* SSS:2:WB    Measure RGB Edge period mode, 1,70sec */
+    i1d_m_rgb_edge_2 = 0x16,		/* SSS:2:WB    Measure RGB edge/period mode, 1.70sec, ret red */
 									/* 2nd return value is not used ? */
     i1d_rdambient    = 0x17,		/* BB:2:BWB    Read Ambient, 1,10sec */
 									/* Returns first B param as first response */
@@ -198,7 +203,7 @@ huey_command(
 	if (isdeb) fprintf(stderr,"huey: Sending cmd '%s' args '%s'",inst_desc(cc), icoms_tohex(in, 7));
 
 	/* Send the command using the control interface */
-	buf[0] = cc;				/* Construct the command */
+	buf[0] = cc;				/* Construct the command == HID report number */
 	memmove(buf + 1, in, 7);
 
 	if (p->icom->is_hid) {
@@ -218,11 +223,19 @@ huey_command(
 			return huey_interp_code((inst *)p, HUEY_COMS_FAIL);
 		}
 	}
-	rv = huey_interp_code((inst *)p, icoms2huey_err(ua));
+	rv = huey_interp_code((inst *)p, icoms2huey_err(ua, 0));
 	if (isdeb) fprintf(stderr," ICOM err 0x%x\n",ua);
-	if (wbytes != 8)
+
+	if (rv == inst_ok && wbytes != 8)
 		rv = huey_interp_code((inst *)p, HUEY_BAD_WR_LENGTH);
+
 	if (rv != inst_ok) {
+		/* Flush any response */
+		if (p->icom->is_hid) {
+			p->icom->hid_read(p->icom, buf, 8, &rbytes, to);
+		} else {
+			p->icom->usb_read(p->icom, 0x81, buf, 8, &rbytes, to);
+		} 
 		p->icom->debug = isdeb;
 		return rv;
 	}
@@ -232,7 +245,6 @@ huey_command(
 
 	if (p->icom->is_hid) {
 		se = p->icom->hid_read(p->icom, buf, 8, &rbytes, to);
-
 	} else {
 		se = p->icom->usb_read(p->icom, 0x81, buf, 8, &rbytes, to);
 	} 
@@ -246,12 +258,11 @@ huey_command(
 			return huey_interp_code((inst *)p, HUEY_COMS_FAIL);
 		}
 	}
-	rv = huey_interp_code((inst *)p, icoms2huey_err(ua));
-	if (rbytes != 8)
+	rv = huey_interp_code((inst *)p, icoms2huey_err(ua, 0));
+	if (rv == inst_ok && rbytes != 8)
 		rv = huey_interp_code((inst *)p, HUEY_BAD_RD_LENGTH);
-	if (rv == inst_ok && buf[1] != cc) {
+	if (rv == inst_ok && buf[1] != cc)
 		rv = huey_interp_code((inst *)p, HUEY_BAD_RET_CMD);
-	}
 		
 	/* Some commands don't use the first response, but need to */
 	/* fetch a second response, with a longer timeout. */
@@ -273,8 +284,8 @@ huey_command(
 				return huey_interp_code((inst *)p, HUEY_COMS_FAIL);
 			}
 		}
-		rv = huey_interp_code((inst *)p, icoms2huey_err(ua));
-		if (rbytes != 8)
+		rv = huey_interp_code((inst *)p, icoms2huey_err(ua, 0));
+		if (rv == inst_ok && rbytes != 8)
 			rv = huey_interp_code((inst *)p, HUEY_BAD_RD_LENGTH);
 		if (rv == inst_ok && buf[1] != cc) {
 			rv = huey_interp_code((inst *)p, HUEY_BAD_RET_CMD);
@@ -540,11 +551,13 @@ huey_wr_int_time(
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - */
 
-/* Take a raw initial (CRT) measurement from the device for a huey */
+/* Take a raw measurement using a given integration time. */
+/* The measureent is the count of (both) edges from the L2V */
+/* over the integration time */
 static inst_code
-huey_take_first_raw_measurement_2(
+huey_freq_measure(
 	huey *p,				/* Object */
-	double rgb[3]			/* Return the RGB values */
+	double rgb[3]			/* Return the RGB edge count values */
 ) {
 	int i;
 	unsigned char ibuf[8];
@@ -570,9 +583,11 @@ huey_take_first_raw_measurement_2(
 	return inst_ok;
 }
 
-/* Take a raw subsequent (CRT + LCD) mesurement from the device for a huey */
+/* Take a raw measurement that returns the number of clocks */
+/* between and initial edge and edgec[] subsequent edges of the L2F. */
+/* Both edges are counted. */
 static inst_code
-huey_take_raw_measurement_2(
+huey_period_measure(
 	huey *p,			/* Object */
 	int edgec[3],		/* Measurement edge count for each channel */
 	double rgb[3]		/* Return the RGB values */
@@ -669,15 +684,15 @@ huey_take_measurement_2(
 	if (p->inited == 0)
 		return huey_interp_code((inst *)p, HUEY_NOT_INITED);
 
-	DBG(("take_measurement_2 called with crtm = %d\n",crtm));
+	DBG((dbgo,"take_measurement_2 called with crtm = %d\n",crtm));
 
 	/* For CRT mode, do an initial set of syncromized measurements */
 	if (crtm) {
 
-		if ((ev = huey_take_first_raw_measurement_2(p, rgb)) != inst_ok)
+		if ((ev = huey_freq_measure(p, rgb)) != inst_ok)
 			return ev;
 
-		DBG(("Raw initial CRT RGB = %f %f %f\n",rgb[0],rgb[1],rgb[2]))
+		DBG((dbgo,"Raw initial CRT RGB = %f %f %f\n",rgb[0],rgb[1],rgb[2]))
 
 		/* Decide whether any channels need re-measuring, */
 		/* and computed cooked values. Threshold is typically 75 */
@@ -685,7 +700,7 @@ huey_take_measurement_2(
 			rem[i] = (rgb[i] <= (0.75 * (double)p->sampno)) ? 1 : 0;
 			rgb[i] = 0.5 * rgb[i] * 1e6/(double)p->int_clocks;
 		}
-		DBG(("Re-measure flags = %d %d %d\n",rem[0],rem[1],rem[2]))
+		DBG((dbgo,"Re-measure flags = %d %d %d\n",rem[0],rem[1],rem[2]))
 	}
 
 	/* If any need re-measuring */
@@ -693,9 +708,9 @@ huey_take_measurement_2(
 		double rgb2[3];
 
 		/* Do a first or second set of measurements */
-		if ((ev = huey_take_raw_measurement_2(p, edgec, rgb2)) != inst_ok)
+		if ((ev = huey_period_measure(p, edgec, rgb2)) != inst_ok)
 			return ev;
-		DBG(("Raw initial/subsequent ecount %d %d %d RGB = %f %f %f\n",
+		DBG((dbgo,"Raw initial/subsequent ecount %d %d %d RGB = %f %f %f\n",
 		     edgec[0], edgec[1], edgec[2], rgb2[0], rgb2[1], rgb2[2]))
 
 		/* Compute adjusted edge count for channels we're remeasuring, */
@@ -718,10 +733,10 @@ huey_take_measurement_2(
 		if (edgec[0] > 1 || edgec[1] > 1 || edgec[2] > 1) {
 			double rgb3[3];		/* 2nd RGB Readings */
 	
-			if ((ev = huey_take_raw_measurement_2(p, edgec, rgb3)) != inst_ok)
+			if ((ev = huey_period_measure(p, edgec, rgb3)) != inst_ok)
 				return ev;
 	
-			DBG(("Raw subsequent2 ecount %d %d %d RGB = %f %f %f\n",
+			DBG((dbgo,"Raw subsequent2 ecount %d %d %d RGB = %f %f %f\n",
 			     edgec[0], edgec[1], edgec[2], rgb3[0], rgb3[1], rgb3[2]))
 
 			/* Average readings if we repeated a measurement with the same threshold */
@@ -738,19 +753,19 @@ huey_take_measurement_2(
 		for (i = 0; i < 3; i++) {
 			if (rem[i]) {
 				rgb[i] = ((double)edgec[i])/(rgb2[i] * 2.0 * p->clk_prd);
-				DBG(("%d after scale = %f\n",i,rgb[i]))
+				DBG((dbgo,"%d after scale = %f\n",i,rgb[i]))
 		
 				rgb[i] -= p->dark_cal[i];		/* Subtract black level */
-				DBG(("%d after sub black = %f\n",i,rgb[i]))
+				DBG((dbgo,"%d after sub black = %f\n",i,rgb[i]))
 		
 				if (rgb[i] < 0.0001)
 					rgb[i] = 0.0001;
-				DBG(("%d after limit min = %f\n",i,rgb[i]))
+				DBG((dbgo,"%d after limit min = %f\n",i,rgb[i]))
 			}
 		}
 	}
 
-	DBG(("Cooked RGB = %f %f %f\n",rgb[0],rgb[1],rgb[2]))
+	DBG((dbgo,"Cooked RGB = %f %f %f\n",rgb[0],rgb[1],rgb[2]))
 	
 	return inst_ok;
 }
@@ -801,12 +816,12 @@ huey_take_amb_measurement(
 	if (p->inited == 0)
 		return huey_interp_code((inst *)p, HUEY_NOT_INITED);
 
-	DBG(("take_amb_measurement_2 called with crtm = %d\n",crtm));
+	DBG((dbgo,"take_amb_measurement_2 called with crtm = %d\n",crtm));
 
 	/* First param is always 3, second is sync mode */
 	if ((ev = huey_take_amb_measurement_1(p, 3, crtm ? 2 : 0, amb, &rb)) != inst_ok)
  		return ev;
-	DBG(("Raw ambient = %f\n",*amb))
+	DBG((dbgo,"Raw ambient = %f\n",*amb))
 	return inst_ok;
 }
 
@@ -875,7 +890,7 @@ huey_take_XYZ_measurement(
 		/* Apply the colorimeter correction matrix */
 		icmMulBy3x3(XYZ, p->ccmat, XYZ);
 	}
-	DBG(("returning XYZ = %f %f %f\n",XYZ[0],XYZ[1],XYZ[2]))
+	DBG((dbgo,"returning XYZ = %f %f %f\n",XYZ[0],XYZ[1],XYZ[2]))
 	return inst_ok;
 }
 
@@ -935,36 +950,36 @@ huey_read_all_regs(
 	/* Serial number */
 	if ((ev = huey_rdreg_word(p, &p->ser_no, 0) ) != inst_ok)
 		return ev;
-	DBG(("serial number = %d\n",p->ser_no))
+	DBG((dbgo,"serial number = %d\n",p->ser_no))
 
 
 	/* LCD/user calibration values */
 	for (i = 0; i < 9; i++) {
 		if ((ev = huey_rdreg_float(p, &p->LCD_cal[i], 4 + 4 * i) ) != inst_ok)
 			return ev;
-		DBG(("LCD/user cal[%d] = %f\n",i,p->LCD_cal[i]))
+		DBG((dbgo,"LCD/user cal[%d] = %f\n",i,p->LCD_cal[i]))
 	}
 	/* LCD/user calibration time */
 	if ((ev = huey_rdreg_word(p, &p->LCD_caltime, 50) ) != inst_ok)
 		return ev;
-	DBG(("LCD/user calibration time = 0x%x = %s\n",p->LCD_caltime, ctime(&p->LCD_caltime)))
+	DBG((dbgo,"LCD/user calibration time = 0x%x = %s\n",p->LCD_caltime, ctime_32(&p->LCD_caltime)))
 
 
 	/* CRT/factory calibration values */
 	for (i = 0; i < 9; i++) {
 		if ((ev = huey_rdreg_float(p, &p->CRT_cal[i], 54 + 4 * i) ) != inst_ok)
 			return ev;
-		DBG(("CRT/factory cal[%d] = %f\n",i,p->CRT_cal[i]))
+		DBG((dbgo,"CRT/factory cal[%d] = %f\n",i,p->CRT_cal[i]))
 	}
 	/* CRT/factory calibration flag */
 	if ((ev = huey_rdreg_word(p, &p->CRT_caltime, 90) ) != inst_ok)
 		return ev;
-	DBG(("CRT/factory flag = 0x%x = %s\n",p->CRT_caltime, ctime(&p->CRT_caltime)))
+	DBG((dbgo,"CRT/factory flag = 0x%x = %s\n",p->CRT_caltime, ctime_32(&p->CRT_caltime)))
 
 
 	/* Hard coded in Huey */
 	p->clk_prd = 1e-6;
-	DBG(("Clock period = %f\n",p->clk_prd))
+	DBG((dbgo,"Clock period = %f\n",p->clk_prd))
 
 	/* Dark current calibration values */
 	for (i = 0; i < 3; i++) {
@@ -973,7 +988,7 @@ huey_read_all_regs(
 				return ev;
 			p->dark_cal[i] = 0.0;
 		}
-		DBG(("darkcal[%d] = %f\n",i,p->dark_cal[i]))
+		DBG((dbgo,"darkcal[%d] = %f\n",i,p->dark_cal[i]))
 	}
 
 	/* Ambient darkcurrent calibration value ? */
@@ -982,7 +997,7 @@ huey_read_all_regs(
 			return ev;
 		p->amb_cal = 0.0;
 	}
-	DBG(("Ambient cal = %f\n",p->amb_cal))
+	DBG((dbgo,"Ambient cal = %f\n",p->amb_cal))
 
 	/* Unlock string */
 	for (i = 0; i < 4; i++) {
@@ -992,12 +1007,12 @@ huey_read_all_regs(
 		p->unlk_string[i] = (char)vv;
 	}
 	p->unlk_string[i] = '\000';
-	DBG(("unlock string = '%s'\n",p->unlk_string))
+	DBG((dbgo,"unlock string = '%s'\n",p->unlk_string))
 
 	/* Read the integration time */
 	if ((ev = huey_rd_int_time(p, &p->int_clocks) ) != inst_ok)
 		return ev;
-	DBG(("Integration time = %d\n",p->int_clocks))
+	DBG((dbgo,"Integration time = %d\n",p->int_clocks))
 
 	if (p->debug) fprintf(stderr,"huey: all registers read OK\n");
 
@@ -1025,7 +1040,7 @@ huey_compute_factors(
 
 	/* clk_prd inversion */
 	p->clk_freq = 1.0/p->clk_prd;
-	DBG(("clk_freq = %f\n",p->clk_freq))
+	DBG((dbgo,"clk_freq = %f\n",p->clk_freq))
 	
 	/* Set some defaults */
 	p->sampno = 100;		/* Minimum sampling count */
@@ -1132,8 +1147,6 @@ huey_init_inst(inst *pp) {
 		if (p->debug) fprintf(stderr,"huey: instrument inited OK\n");
 	}
 
-	p->itype = instHuey;
-
 	/* Flash the LEDs, just cos we can! */
 	if ((ev = huey_set_LEDs(p, 0x1)) != inst_ok)
 		return ev;
@@ -1173,11 +1186,16 @@ ipatch *val) {		/* Pointer to instrument patch value */
 	int user_trig = 0;
 	int rv = inst_protocol_error;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (p->trig == inst_opt_trig_keyb) {
 		int se;
 		if ((se = icoms_poll_user(p->icom, 1)) != ICOM_TRIG) {
 			/* Abort, term or command */
-			return huey_interp_code((inst *)p, icoms2huey_err(se));
+			return huey_interp_code((inst *)p, icoms2huey_err(se, 1));
 		}
 		user_trig = 1;
 		if (p->trig_return)
@@ -1209,6 +1227,11 @@ double mtx[3][3]
 ) {
 	huey *p = (huey *)pp;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (mtx == NULL)
 		icmSetUnity3x3(p->ccmat);
 	else
@@ -1224,6 +1247,11 @@ double mtx[3][3]
 inst_cal_type huey_needs_calibration(inst *pp) {
 	huey *p = (huey *)pp;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	return inst_ok;
 }
 
@@ -1237,6 +1265,11 @@ char id[CALIDLEN]		/* Condition identifier (ie. white reference ID) */
 	huey *p = (huey *)pp;
 	int rv = 0;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	id[0] = '\000';
 
 	return inst_unsupported;
@@ -1398,8 +1431,8 @@ inst_capability huey_capabilities(inst *pp) {
 
 	rv = inst_emis_spot
 	   | inst_emis_disp
-	   | inst_emis_disp_crt
-	   | inst_emis_disp_lcd
+	   | inst_emis_disptype
+	   | inst_emis_disptypem
 	   | inst_colorimeter
 	   | inst_ccmx
 	   | inst_emis_ambient
@@ -1421,12 +1454,65 @@ inst2_capability huey_capabilities2(inst *pp) {
 	return rv;
 }
 
+inst_disptypesel huey_disptypesel[3] = {
+	{
+		1,
+		"c",
+		"Huey: CRT display",
+		1
+	},
+	{
+		2,
+		"l",
+		"Huey: LCD display",
+		0
+	},
+	{
+		0,
+		"",
+		"",
+		-1
+	}
+};
+
+/* Get mode and option details */
+static inst_code huey_get_opt_details(
+inst *pp,
+inst_optdet_type m,	/* Requested option detail type */
+...) {				/* Status parameters */                             
+	huey *p = (huey *)pp;
+	inst_code rv = inst_ok;
+
+	if (m == inst_optdet_disptypesel) {
+		va_list args;
+		int *pnsels;
+		inst_disptypesel **psels;
+
+		va_start(args, m);
+		pnsels = va_arg(args, int *);
+		psels = va_arg(args, inst_disptypesel **);
+		va_end(args);
+
+		*pnsels = 2;
+		*psels = huey_disptypesel;
+		
+		return inst_ok;
+	}
+
+	return inst_unsupported;
+}
+
 /* Set device measurement mode */
 inst_code huey_set_mode(inst *pp, inst_mode m)
 {
 	huey *p = (huey *)pp;
 	inst_mode mm;		/* Measurement mode */
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	/* The measurement mode portion of the mode */
 	mm = m & inst_mode_measurement_mask;
 
@@ -1455,17 +1541,33 @@ huey_set_opt_mode(inst *pp, inst_opt_mode m, ...)
 	huey *p = (huey *)pp;
 	inst_code ev = inst_ok;
 
-	/* Select CRT/LCD */
-	if (m == inst_opt_disp_crt) {
-		if (p->crt == 0)
-		p->crt = 1;
-		return inst_ok;
-	} else if (m == inst_opt_disp_lcd) {
-		if (p->crt != 0)
-		p->crt = 0;
-		return inst_ok;
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
 
+	/* Set the display type */
+	if (m == inst_opt_disp_type) {
+		va_list args;
+		int ix;
+
+		va_start(args, m);
+		ix = va_arg(args, int);
+		va_end(args);
+
+		if (ix == 1) {
+			if (p->crt == 0)
+			p->crt = 1;
+			return inst_ok;
+		} else if (ix == 2) {
+			if (p->crt != 0)
+			p->crt = 0;
+			return inst_ok;
+		} else {
+			return inst_unsupported;
+		}
 	}
+
 	/* Record the trigger mode */
 	if (m == inst_opt_trig_prog
 	 || m == inst_opt_trig_keyb) {
@@ -1513,7 +1615,7 @@ huey_set_opt_mode(inst *pp, inst_opt_mode m, ...)
 }
 
 /* Constructor */
-extern huey *new_huey(icoms *icom, int debug, int verb)
+extern huey *new_huey(icoms *icom, instType itype, int debug, int verb)
 {
 	huey *p;
 	if ((p = (huey *)calloc(sizeof(huey),1)) == NULL)
@@ -1533,6 +1635,7 @@ extern huey *new_huey(icoms *icom, int debug, int verb)
 	p->init_inst         = huey_init_inst;
 	p->capabilities      = huey_capabilities;
 	p->capabilities2     = huey_capabilities2;
+	p->get_opt_details   = huey_get_opt_details;
 	p->set_mode          = huey_set_mode;
 	p->set_opt_mode      = huey_set_opt_mode;
 	p->read_sample       = huey_read_sample;
@@ -1542,7 +1645,7 @@ extern huey *new_huey(icoms *icom, int debug, int verb)
 	p->interp_error      = huey_interp_error;
 	p->del               = huey_del;
 
-	p->itype = instUnknown;		/* Until initalisation */
+	p->itype = itype;
 
 	return p;
 }
